@@ -1,19 +1,13 @@
 // Copyright 2013 The Servo Project Developers. See the LICENSE
 // file at the top-level directory of this distribution.
 //
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
+// Licensed under the the MIT license. This file may not be
+// copied, modified, or distributed except according to those terms.
 
 #![allow(non_upper_case_globals)]
 #![allow(non_camel_case_types)]
 #![allow(non_snake_case)]
 
-#[macro_use]
-extern crate const_cstr;
-
 use std::os::raw::{c_char, c_double, c_int, c_uchar, c_uint, c_ushort, c_void};
 
 pub use dlib::ffi_dispatch;
@@ -56,6 +50,10 @@ pub const FcTypeLangSet: u32 = 8_u32;
 pub type FcType = enum__FcType;
 
 pub mod constants {
+    use std::ffi::CStr;
+
+    use cstr::cstr;
+
     use super::c_int;
 
     pub const FC_WEIGHT_THIN: c_int = 0;
@@ -113,67 +111,67 @@ pub mod constants {
     pub const FC_LCD_LEGACY: c_int = 3;
 
     pub const FC_CHARSET_MAP_SIZE: c_int = 8;
+    pub const FC_CHARSET_DONE: u32 = u32::MAX;
     pub const FC_UTF8_MAX_LEN: c_int = 6;
 
-    const_cstr! {
-        pub FC_FAMILY = "family";
-        pub FC_STYLE = "style";
-        pub FC_SLANT = "slant";
-        pub FC_WEIGHT = "weight";
-        pub FC_SIZE = "size";
-        pub FC_ASPECT = "aspect";
-        pub FC_PIXEL_SIZE = "pixelsize";
-        pub FC_SPACING = "spacing";
-        pub FC_FOUNDRY = "foundry";
-        pub FC_ANTIALIAS = "antialias";
-        pub FC_HINTING = "hinting";
-        pub FC_HINT_STYLE = "hintstyle";
-        pub FC_VERTICAL_LAYOUT = "verticallayout";
-        pub FC_AUTOHINT = "autohint";
-        pub FC_GLOBAL_ADVANCE = "globaladvance";
-        pub FC_WIDTH = "width";
-        pub FC_FILE = "file";
-        pub FC_INDEX = "index";
-        pub FC_FT_FACE = "ftface";
-        pub FC_RASTERIZER = "rasterizer";
-        pub FC_OUTLINE = "outline";
-        pub FC_SCALABLE = "scalable";
-        pub FC_COLOR = "color";
-        pub FC_VARIABLE = "variable";
-        pub FC_SCALE = "scale";
-        pub FC_SYMBOL = "symbol";
-        pub FC_DPI = "dpi";
-        pub FC_RGBA = "rgba";
-        pub FC_MINSPACE = "minspace";
-        pub FC_SOURCE = "source";
-        pub FC_CHARSET = "charset";
-        pub FC_LANG = "lang";
-        pub FC_FONTVERSION = "fontversion";
-        pub FC_FULLNAME = "fullname";
-        pub FC_FAMILYLANG = "familylang";
-        pub FC_STYLELANG = "stylelang";
-        pub FC_FULLNAMELANG = "fullnamelang";
-        pub FC_CAPABILITY = "capability";
-        pub FC_FONTFORMAT = "fontformat";
-        pub FC_EMBOLDEN = "embolden";
-        pub FC_EMBEDDED_BITMAP = "embeddedbitmap";
-        pub FC_DECORATIVE = "decorative";
-        pub FC_LCD_FILTER = "lcdfilter";
-        pub FC_FONT_FEATURES = "fontfeatures";
-        pub FC_FONT_VARIATIONS = "fontvariations";
-        pub FC_NAMELANG = "namelang";
-        pub FC_PRGNAME = "prgname";
-        pub FC_HASH = "hash";
-        pub FC_POSTSCRIPT_NAME = "postscriptname";
-        pub FC_FONT_HAS_HINT = "fonthashint";
-        pub FC_CACHE_SUFFIX = ".cache-";
-        pub FC_DIR_CACHE_FILE = "fonts.cache-";
-        pub FC_USER_CACHE_FILE = ".fonts.cache-";
-        pub FC_CHARWIDTH = "charwidth";
-        pub FC_CHAR_WIDTH = "charwidth";
-        pub FC_CHAR_HEIGHT = "charheight";
-        pub FC_MATRIX = "matrix";
-    }
+    pub const FC_FAMILY: &CStr = cstr!("family");
+    pub const FC_STYLE: &CStr = cstr!("style");
+    pub const FC_SLANT: &CStr = cstr!("slant");
+    pub const FC_WEIGHT: &CStr = cstr!("weight");
+    pub const FC_SIZE: &CStr = cstr!("size");
+    pub const FC_ASPECT: &CStr = cstr!("aspect");
+    pub const FC_PIXEL_SIZE: &CStr = cstr!("pixelsize");
+    pub const FC_SPACING: &CStr = cstr!("spacing");
+    pub const FC_FOUNDRY: &CStr = cstr!("foundry");
+    pub const FC_ANTIALIAS: &CStr = cstr!("antialias");
+    pub const FC_HINTING: &CStr = cstr!("hinting");
+    pub const FC_HINT_STYLE: &CStr = cstr!("hintstyle");
+    pub const FC_VERTICAL_LAYOUT: &CStr = cstr!("verticallayout");
+    pub const FC_AUTOHINT: &CStr = cstr!("autohint");
+    pub const FC_GLOBAL_ADVANCE: &CStr = cstr!("globaladvance");
+    pub const FC_WIDTH: &CStr = cstr!("width");
+    pub const FC_FILE: &CStr = cstr!("file");
+    pub const FC_INDEX: &CStr = cstr!("index");
+    pub const FC_FT_FACE: &CStr = cstr!("ftface");
+    pub const FC_RASTERIZER: &CStr = cstr!("rasterizer");
+    pub const FC_OUTLINE: &CStr = cstr!("outline");
+    pub const FC_SCALABLE: &CStr = cstr!("scalable");
+    pub const FC_COLOR: &CStr = cstr!("color");
+    pub const FC_VARIABLE: &CStr = cstr!("variable");
+    pub const FC_SCALE: &CStr = cstr!("scale");
+    pub const FC_SYMBOL: &CStr = cstr!("symbol");
+    pub const FC_DPI: &CStr = cstr!("dpi");
+    pub const FC_RGBA: &CStr = cstr!("rgba");
+    pub const FC_MINSPACE: &CStr = cstr!("minspace");
+    pub const FC_SOURCE: &CStr = cstr!("source");
+    pub const FC_CHARSET: &CStr = cstr!("charset");
+    pub const FC_LANG: &CStr = cstr!("lang");
+    pub const FC_FONTVERSION: &CStr = cstr!("fontversion");
+    pub const FC_FULLNAME: &CStr = cstr!("fullname");
+    pub const FC_FAMILYLANG: &CStr = cstr!("familylang");
+    pub const FC_STYLELANG: &CStr = cstr!("stylelang");
+    pub const FC_FULLNAMELANG: &CStr = cstr!("fullnamelang");
+    pub const FC_CAPABILITY: &CStr = cstr!("capability");
+    pub const FC_FONTFORMAT: &CStr = cstr!("fontformat");
+    pub const FC_EMBOLDEN: &CStr = cstr!("embolden");
+    pub const FC_EMBEDDED_BITMAP: &CStr = cstr!("embeddedbitmap");
+    pub const FC_DECORATIVE: &CStr = cstr!("decorative");
+    pub const FC_LCD_FILTER: &CStr = cstr!("lcdfilter");
+    pub const FC_FONT_FEATURES: &CStr = cstr!("fontfeatures");
+    pub const FC_FONT_VARIATIONS: &CStr = cstr!("fontvariations");
+    pub const FC_NAMELANG: &CStr = cstr!("namelang");
+    pub const FC_PRGNAME: &CStr = cstr!("prgname");
+    pub const FC_HASH: &CStr = cstr!("hash");
+    pub const FC_POSTSCRIPT_NAME: &CStr = cstr!("postscriptname");
+    pub const FC_FONT_HAS_HINT: &CStr = cstr!("fonthashint");
+    pub const FC_CACHE_SUFFIX: &CStr = cstr!(".cache-");
+    pub const FC_DIR_CACHE_FILE: &CStr = cstr!("fonts.cache-");
+    pub const FC_USER_CACHE_FILE: &CStr = cstr!(".fonts.cache-");
+    pub const FC_CHARWIDTH: &CStr = cstr!("charwidth");
+    pub const FC_CHAR_WIDTH: &CStr = cstr!("charwidth");
+    pub const FC_CHAR_HEIGHT: &CStr = cstr!("charheight");
+    pub const FC_MATRIX: &CStr = cstr!("matrix");
+    pub const FC_ORDER: &CStr = cstr!("order");
 }
 
 #[repr(C)]
@@ -400,6 +398,8 @@ dlib::external_library!(Fc, "fontconfig",
 
         fn FcCharSetAddChar(*mut FcCharSet, FcChar32) -> FcBool,
 
+        fn FcCharSetDelChar(*mut FcCharSet, FcChar32) -> FcBool,
+
         fn FcCharSetCopy(*mut FcCharSet) -> *mut FcCharSet,
 
         fn FcCharSetEqual(*const FcCharSet, *const FcCharSet) -> FcBool,
@@ -775,6 +775,11 @@ dlib::external_library!(Fc, "fontconfig",
             *mut *mut FcLangSet
         ) -> FcResult,
 
+        // The last argument is a pointer to a FreeType Face object (`FT_Face *`)
+        //
+        // <https://freetype.org/freetype2/docs/reference/ft2-base_interface.html#ft_face>
+        fn FcPatternGetFTFace(*mut FcPattern, *const c_char, c_int, *mut *mut c_void) -> FcResult,
+
         // fn FcPatternVaBuild(*mut FcPattern, *mut __va_list_tag) -> *mut FcPattern,
 
         fn FcPatternFormat(*mut FcPattern, *const FcChar8) -> *mut FcChar8,
